import { VoidNode } from './primitive.js';
import { Simplify } from './type-utils';
import { AnyBaseNode, BaseNode, Infer, InferNodeArray } from './types';
export type AnyObjectShape = {
    [key: string]: AnyBaseNode;
};
export interface ObjectNode<Shape extends AnyObjectShape> extends BaseNode<'object'> {
    readonly _type: Simplify<{
        [Property in keyof Shape as undefined extends Infer<Shape[Property]> ? never : Property]: Infer<Shape[Property]>;
    } & {
        [Property in keyof Shape as undefined extends Infer<Shape[Property]> ? Property : never]?: Infer<Shape[Property]>;
    }>;
    shape: Shape;
}
export type AnyObjectNode = ObjectNode<{
    [key: string]: any;
}>;
export declare const object: <TShape extends AnyObjectShape>(shape: TShape) => ObjectNode<TShape>;
export interface ArrayNode<Type extends AnyBaseNode> extends BaseNode<'array'> {
    readonly _type: Infer<Type>[];
    type: Type;
}
export type AnyArrayNode = ArrayNode<any>;
export declare const array: <TNode extends AnyBaseNode>(type: TNode) => ArrayNode<TNode>;
export interface IteratorNode<Type extends AnyBaseNode> extends BaseNode<'iterator'> {
    readonly _type: Infer<Type>[];
    type: Type;
}
export type AnyIteratorNode = IteratorNode<any>;
export declare const iterator: <TNode extends AnyBaseNode>(type: TNode) => IteratorNode<TNode>;
export interface SetNode<Type extends AnyBaseNode> extends BaseNode<'set'> {
    readonly _type: Set<Infer<Type>>;
    type: Type;
}
export type AnySetNode = SetNode<any>;
export declare const set: <TNode extends AnyBaseNode>(type: TNode) => SetNode<TNode>;
export interface AnyRecordKeyNode extends AnyBaseNode {
    readonly _type: string | number;
}
export interface RecordNode<KeyNode extends AnyBaseNode, ValueNode extends AnyBaseNode> extends BaseNode<'record'> {
    readonly _type: Record<Infer<KeyNode>, Infer<ValueNode>>;
    key: KeyNode;
    value: ValueNode;
}
export type AnyRecordNode = RecordNode<any, any>;
export declare const record: <KeyNode extends AnyRecordKeyNode, ValueNode extends AnyBaseNode>(key: KeyNode, value: ValueNode) => RecordNode<KeyNode, ValueNode>;
export interface MapNode<KeyNode extends AnyBaseNode, ValueNode extends AnyBaseNode> extends BaseNode<'map'> {
    readonly _type: Map<Infer<KeyNode>, Infer<ValueNode>>;
    key: KeyNode;
    value: ValueNode;
}
export type AnyMapNode = MapNode<any, any>;
export declare const map: <KeyNode extends AnyBaseNode, ValueNode extends AnyBaseNode>(key: KeyNode, value: ValueNode) => MapNode<KeyNode, ValueNode>;
export interface FunctionNode<TArguments extends AnyBaseNode[], TReturn extends AnyBaseNode> extends BaseNode<'function'> {
    readonly _type: (...arguments_: InferNodeArray<TArguments>) => Infer<TReturn>;
    arguments: TArguments;
    return: TReturn;
}
export type AnyFunctionNode = FunctionNode<any[], any>;
export declare const func: <const TArguments extends AnyBaseNode[] = [], TReturn extends AnyBaseNode = VoidNode>(options?: {
    arguments?: [...TArguments] | undefined;
    return?: TReturn | undefined;
}) => FunctionNode<TArguments, TReturn>;
export interface UnionNode<Types extends AnyBaseNode[]> extends BaseNode<'union'> {
    readonly _type: {
        [K in keyof Types]: Infer<Types[K]>;
    }[number];
    types: Types;
}
export type AnyUnionNode = UnionNode<any>;
export declare const union: <TNode extends AnyBaseNode, TNodes extends AnyBaseNode[]>(nodes: [TNode, ...TNodes]) => UnionNode<[TNode, ...TNodes]>;
export interface TupleNode<Types extends AnyBaseNode[]> extends BaseNode<'tuple'> {
    readonly _type: InferNodeArray<Types>;
    types: Types;
}
export type AnyTupleNode = TupleNode<any>;
export declare const tuple: <TNodes extends AnyBaseNode[]>(nodes: [...TNodes]) => TupleNode<TNodes>;
type InferIntersection<TNodes extends AnyBaseNode[], InferredType = Infer<TNodes[0]>> = TNodes extends [
    infer Head extends AnyBaseNode,
    ...infer Rest extends AnyBaseNode[]
] ? InferIntersection<Rest, InferredType & Infer<Head>> : InferredType;
export interface IntersectionNode<Types extends AnyBaseNode[]> extends BaseNode<'intersection'> {
    readonly _type: InferIntersection<Types>;
    types: Types;
}
export type AnyIntersectionNode = IntersectionNode<any>;
export declare const intersection: <TNodes extends AnyBaseNode[]>(nodes: [...TNodes]) => IntersectionNode<TNodes>;
export type AnyFormDataShape = {
    [name: string]: AnyBaseNode;
};
export interface FormDataNode<Shape extends AnyFormDataShape> extends BaseNode<'formData'> {
    readonly _type: Simplify<{
        [Property in keyof Shape as undefined extends Infer<Shape[Property]> ? never : Property]: Infer<Shape[Property]>;
    } & {
        [Property in keyof Shape as undefined extends Infer<Shape[Property]> ? Property : never]?: Infer<Shape[Property]>;
    }>;
    shape?: Shape;
}
export type AnyFormDataNode = FormDataNode<{
    [key: string]: any;
}>;
export declare const formData: <TShape extends AnyFormDataShape>(shape?: TShape | undefined) => FormDataNode<TShape>;
export interface CustomNode<TType, TArguments extends AnyBaseNode[]> extends BaseNode<'custom'> {
    readonly _type: {
        type: TType;
        arguments: InferNodeArray<TArguments>;
    };
    readonly name: string;
    readonly arguments?: TArguments;
}
export type AnyCustomNode = CustomNode<any, any[]>;
export declare const custom: <TType, TArguments extends AnyBaseNode[]>(name: string, arguments_?: TArguments | undefined) => CustomNode<TType, TArguments>;
export {};
//# sourceMappingURL=complex.d.ts.map