import {
  any,
  array,
  bigint,
  blob,
  boolean,
  circular,
  custom,
  date,
  defineNode,
  definitionReference,
  file,
  formData,
  func,
  intersection,
  iterator,
  literal,
  map,
  named,
  never,
  nullType,
  number,
  object,
  record,
  regexp,
  set,
  string,
  symbol,
  tuple,
  undefinedType,
  union,
  unknown,
  voidType
} from "./chunk-JWRHR5EW.js";

// src/composed.ts
var nullable = (type) => {
  return union([type, nullType()]);
};
var optional = (type) => {
  return union([type, undefinedType()]);
};

// src/utils.ts
var mapMultipleInnerNode = (node, mapper) => node.types.map((type) => mapper(type));
var mapObjectNode = (node, mapper) => Object.entries(node.shape).map(
  ([key, value]) => [key, mapper(value)]
);
var serializePrimitive = (value) => {
  switch (typeof value) {
    case "string": {
      return JSON.stringify(value);
    }
    case "number": {
      return value.toString();
    }
    case "boolean": {
      return value === true ? "true" : "false";
    }
    case "bigint": {
      return `${value.toString()}n`;
    }
  }
};
var isNodeOptional = (node) => {
  if (node.typeName === "undefined") {
    return true;
  }
  if (node.typeName === "union") {
    return node.types.some((type) => isNodeOptional(type));
  }
  return false;
};
var unionIfMultiple = (nodes) => {
  if (nodes.length === 1) {
    return nodes[0];
  }
  return union(nodes);
};
var UnsupportedError = class extends Error {
  constructor(integration, feature) {
    super(`the onetyped ${integration} integration does not support ${feature}`);
    this.name = "UnsupportedError";
    this.integration = integration;
    this.feature = feature;
    Object.setPrototypeOf(this, UnsupportedError.prototype);
  }
};

// src/print-node.ts
function printNode(node, options) {
  const printedNode = printNodeInternal(node, options);
  return printedNode;
}
function printNodeInternal(node, options) {
  const { indent = 2, depth = 0 } = options ?? {};
  const baseIndentString = " ".repeat(indent * depth);
  const nestedIndentString = " ".repeat(indent * (depth + 1));
  switch (node.typeName) {
    case "any":
    case "boolean":
    case "never":
    case "null":
    case "number":
    case "string":
    case "undefined":
    case "unknown":
    case "void": {
      return node.typeName;
    }
    case "bigint": {
      return "BigInt";
    }
    case "blob": {
      return "Blob";
    }
    case "date": {
      return "Date";
    }
    case "file": {
      return "File";
    }
    case "regexp": {
      return "RegExp";
    }
    case "array": {
      return `(${printNodeInternal(node.type, options)})[]`;
    }
    case "intersection": {
      return "(" + node.types.map((t) => printNodeInternal(t, options)).join(" & ") + ")";
    }
    case "literal": {
      return JSON.stringify(node.type);
    }
    case "map": {
      return `Map<${printNodeInternal(node.key, options)}, ${printNodeInternal(
        node.value,
        options
      )}>`;
    }
    case "record": {
      return `Record<${printNodeInternal(
        node.key,
        options
      )}, ${printNodeInternal(node.value, options)}>`;
    }
    case "set": {
      return `Set<${printNodeInternal(node.type, options)}>`;
    }
    case "tuple": {
      return `[${node.types.map((t) => printNodeInternal(t, options)).join(", ")}]`;
    }
    case "union": {
      return "(" + node.types.map((t) => printNodeInternal(t, options)).join(" | ") + ")";
    }
    case "iterator": {
      return `Iterator<${printNodeInternal(node.type, options)}>`;
    }
    case "symbol": {
      return `Symbol(${node.name})`;
    }
    case "formData": {
      if (!node.shape) {
        return "FormData";
      }
      const printedShape = Object.entries(node.shape).map(
        ([name, n]) => {
          return `${nestedIndentString}${name}: ${printNodeInternal(n, {
            ...options,
            depth: depth + 1
          })}`;
        }
      );
      return `FormData {
${printedShape.join(";\n")};
${baseIndentString}}`;
    }
    case "object": {
      const printedShape = Object.entries(node.shape).map(
        ([name, n]) => {
          return `${nestedIndentString}${name}: ${printNodeInternal(n, {
            ...options,
            depth: depth + 1
          })}`;
        }
      );
      return `{
${printedShape.join(";\n")};
${baseIndentString}}`;
    }
    case "function": {
      const printedArguments = node.arguments.map(
        (a, index) => `args_${index}: ${printNodeInternal(a, options)}`
      ).join(", ");
      return `(${printedArguments}) => ${printNodeInternal(
        node.return,
        options
      )};`;
    }
    case "custom": {
      if (!node.arguments) {
        return node.name;
      }
      return `${node.name}<${node.arguments.map((a) => printNodeInternal(a, options)).join(", ")}>`;
    }
    case "named": {
      return node.name;
    }
    case "definitionReference": {
      return `DefinitionReference<${node.text}>`;
    }
    case "circular": {
      return "[Circular]";
    }
  }
}
export {
  UnsupportedError,
  any,
  array,
  bigint,
  blob,
  boolean,
  circular,
  custom,
  date,
  defineNode,
  definitionReference,
  file,
  formData,
  func,
  intersection,
  isNodeOptional,
  iterator,
  literal,
  map,
  mapMultipleInnerNode,
  mapObjectNode,
  named,
  never,
  nullType,
  nullable,
  number,
  object,
  optional,
  printNode,
  record,
  regexp,
  serializePrimitive,
  set,
  string,
  symbol,
  tuple,
  undefinedType,
  union,
  unionIfMultiple,
  unknown,
  voidType
};
